# -*- coding: utf-8 -*-
"""
    Copyright (C) 2013 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
if sys.version_info > (2, 6) and sys.version_info < (2, 8):
    import ConfigParser as configparser
elif sys.version_info > (3, 0):
    import configpaser
import utils


def check_config(filename):
    """Check configuration file of backup2swift

    Argument:

        filename: config file path (default is ~/.bu2sw.conf)
    """
    try:
        conf = configparser.SafeConfigParser(allow_no_value=False)
    except TypeError as error:
        msg = "__init__() got an unexpected keyword argument 'allow_no_value'"
        if str(error) == msg:
            # for argparse using python 2.6
            conf = configparser.SafeConfigParser()
        else:
            utils.logging(3, error)
    conf.read(filename)
    try:
        auth_url = conf.get('swift', 'auth_url')
        username = conf.get('swift', 'username')
        password = conf.get('swift', 'password')
        rotate_limit = conf.get('backup', 'rotate_limit')
    except (configparser.NoSectionError, configparser.NoOptionError) as error:
        # syslog.ERR is 3
        utils.logging(3, error)
    try:
        if conf.get('swift', 'ignore_verify_ssl_certification') == 'True':
            verify = False
        else:
            verify = True
    except (configparser.NoSectionError, configparser.NoOptionError):
        verify = True
    return auth_url, username, password, rotate_limit, verify



# -*- coding: utf-8 -*-
"""
    Copyright (C) 2013 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import unittest
from mock import patch
import swiftsc.client
import sys
import os.path
sys.path.append(os.path.abspath('src'))
import backup2swift.backup as b
import test_vars as v


class BackupTests(unittest.TestCase):

    @patch('swiftsc.client.retrieve_token', return_value=(v.token, v.s_url))
    def setUp(self, m):
        self.b = b.Backup(v.auth_url, v.username, v.password)

    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.list_objects', return_value=v.objects)
    @patch('swiftsc.client.create_object', return_value=201)
    def test_backup(self, m1, m2, m3):
        self.assertEqual(self.b.backup("."), True)

    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.create_container', return_value=201)
    @patch('swiftsc.client.list_objects', return_value=v.objects)
    @patch('swiftsc.client.create_object', return_value=201)
    def test_backup_file_with_create_cont(self, m1, m2, m3, m4):
        self.assertEqual(self.b.backup_file("examples/bu2sw.conf"), True)

    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.create_container', return_value=202)
    @patch('swiftsc.client.list_objects', return_value=v.objects)
    @patch('swiftsc.client.create_object', return_value=201)
    def test_backup_file_with_created_cont(self, m1, m2, m3, m4):
        self.assertEqual(self.b.backup_file("examples/bu2sw.conf"), True)

    @unittest.expectedFailure
    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.create_container', return_value=400)
    def test_backup_faile_fail_create_cont(self, m1, m2):
        self.assertEqual(self.b.backup_file("examples/bu2sw.conf"), True)

    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.create_container', return_value=202)
    @patch('swiftsc.client.list_objects', return_value=v.objects)
    @patch('swiftsc.client.create_object', return_value=201)
    def test_backup_file_create_object(self, m1, m2, m3, m4):
        self.assertEqual(self.b.backup_file("examples/bu2sw.conf"), True)

    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.create_container', return_value=201)
    @patch('swiftsc.client.list_objects', return_value=v.objects)
    @patch('swiftsc.client.create_object', return_value=202)
    def test_backup_file_override(self, m1, m2, m3, m4):
        self.assertEqual(self.b.backup_file("examples/bu2sw.conf"), True)

    @unittest.expectedFailure
    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.create_container', return_value=201)
    @patch('swiftsc.client.list_objects', return_value=v.objects)
    @patch('swiftsc.client.create_object', return_value=400)
    def test_backup_file_fail(self, m1, m2, m3, m4):
        self.assertEqual(self.b.backup_file("examples/bu2sw.conf"), True)

    @patch('swiftsc.client.copy_object', return_value=201)
    @patch('swiftsc.client.create_object', return_value=201)
    @patch('swiftsc.client.delete_object', return_value=204)
    def test_rotate(self, m1, m2, m3):
        self.assertEqual(self.b.rotate(v.test_file, v.object_name,
                                       v.objects_name_l), True)

    @unittest.expectedFailure
    @patch('swiftsc.client.copy_object', return_value=400)
    def test_rotate_fail_copy(self, m1):
        self.assertEqual(self.b.rotate(v.test_file, v.object_name,
                                       v.objects_name_l), True)

    @unittest.expectedFailure
    @patch('swiftsc.client.copy_object', return_value=201)
    @patch('swiftsc.client.create_object', return_value=400)
    def test_rotate_fail_create(self, m1, m2):
        self.assertEqual(self.b.rotate(v.test_file, v.object_name,
                                       v.objects_name_l), True)

    # ToDo should add raise exception when delete object?
    @patch('swiftsc.client.copy_object', return_value=201)
    @patch('swiftsc.client.create_object', return_value=201)
    @patch('swiftsc.client.delete_object', return_value=400)
    def test_rotate_fail_delete(self, m1, m2, m3):
        self.assertEqual(self.b.rotate(v.test_file, v.object_name,
                                       v.objects_name_l), True)

    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.list_objects', return_value=v.objects)
    def test_retrieve_backup_data_list(self, m1, m2):
        self.assertEqual(self.b.retrieve_backup_data_list(),
                         v.objects_name_l)
        self.assertEqual(self.b.retrieve_backup_data_list(True),
                         v.objects)

    @patch('swiftsc.client.is_container', return_value=False)
    def test_retrieve_backup_data_list(self, m):
        self.assertEqual(self.b.retrieve_backup_data_list(),
                         [])

    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.is_object', return_value=True)
    @patch('swiftsc.client.delete_object', return_value=204)
    def test_delete_backup_data(self, m1, m2, m3):
        self.assertEqual(self.b.delete_backup_data("dummy"), True)

    @unittest.expectedFailure
    @patch('swiftsc.client.is_container', return_value=False)
    def test_delete_backup_data_without_container(self, m):
        self.assertEqual(self.b.delete_backup_data("dummy"), True)

    @unittest.expectedFailure
    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.is_object', return_value=False)
    def test_delete_backup_data_without_object(self, m1, m2):
        self.assertEqual(self.b.delete_backup_data("dummy"), True)

    @unittest.expectedFailure
    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.is_object', return_value=True)
    @patch('swiftsc.client.delete_object', return_value=False)
    def test_delete_backup_data_runtime_error(self, m1, m2, m3):
        self.assertEqual(self.b.delete_backup_data("dummy"), True)

    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.is_object', return_value=True)
    @patch('swiftsc.client.retrieve_object', return_value=(True, ''))
    def test_retrieve_backup_data(self, m1, m2, m3):
        self.assertEqual(self.b.retrieve_backup_data("dummy"), None)

    @unittest.expectedFailure
    @patch('swiftsc.client.is_container', return_value=False)
    def test_retrieve_backup_data_without_container(self, m):
        self.assertEqual(self.b.retrieve_backup_data("dummy"), None)

    @unittest.expectedFailure
    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.is_object', return_value=False)
    def test_retrieve_backup_data_without_object(self, m1, m2):
        self.assertEqual(self.b.retrieve_backup_data("dummy"), None)

    @unittest.expectedFailure
    @patch('swiftsc.client.is_container', return_value=True)
    @patch('swiftsc.client.is_object', return_value=True)
    @patch('swiftsc.client.retrieve_object', return_value=(False, ''))
    def test_retrieve_backup_data(self, m1, m2, m3):
        self.assertEqual(self.b.retrieve_backup_data("dummy"), None)



# -*- coding: utf-8 -*-
"""
    Copyright (C) 2013 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import syslog
import sys
from __init__ import __NAME__


def logging(priority, message):
    """

    Arguments:
        priority: syslog priority
        message: log message
    """
    syslog.openlog(__NAME__, syslog.LOG_PID, syslog.LOG_LOCAL0)
    syslog.syslog(priority, str(message))
    syslog.closelog()
    print(message)
    exit(1)


def list_data(data):
    """

    Argument:
        data: list of data
    """
    for i in data:
        if isinstance(i, unicode):
            print(i)
        elif isinstance(i, dict):
            pretty_print(i.keys(), data)
            break


def pretty_print(header, rows):
    """

    Arguments:
        header: list of header data
        rows:   list of data
    """
    # formatting header
    header.sort()
    header.reverse()
    # retrieve column width from header
    col_width_l = [len(i) for i in header]

    # retrieve column width from data
    col_width_l = get_columns_width(col_width_l, header, rows)

    # print formatly
    print_header(col_width_l, header)
    for row in rows:
        print generate_row_s(row, col_width_l, header)
    print_footer(col_width_l)


def get_columns_width(columns_width, header, rows):
    """

    Arguments:
        columns_width: list of columns string length
        header:        list of header data
        rows:          list of data
    """
    for row in rows:
        for i, key in enumerate(header):
            value = str(row.get(key))
            if columns_width[i] <= len(value):
                columns_width[i] = len(value)
            else:
                columns_width[i] = columns_width[i]
    return columns_width


def print_header(columns_width, header):
    """

    Arguments:
        columns_width: list of columns string length
        header:        list of header data
    """
    border = '+'
    for col_width in columns_width:
        border += "-" * (col_width + 1) + '-+'

    print("%s" % border)
    print(generate_row_s(header, columns_width))
    print("%s" % border)


def print_footer(columns_width):
    """

    Argument:
        columns_width: list of columns string length
    """
    border = '+'
    for i, col_width in enumerate(columns_width):
        border += "-" * (col_width + 1) + '-+'
    print("%s" % border)


def generate_row_s(row, columns_width, header=None):
    """

    Arguments:
        row:           data of row
        columns_width: list of columns string length
        header:        list of header data
    """
    row_s = ''

    if header:
        for i, key in enumerate(header):
            column = str(row.get(key))
            row_s += ('| ' + column + ' ' *
                      (columns_width[i] - len(column) + 1))
        row_s += '|'
    else:
        for i, value in enumerate(row):
            column = str(value)
            row_s += ('| ' + column + ' ' *
                      (columns_width[i] - len(column) + 1))
        row_s += '|'
    return row_s



# -*- coding: utf-8 -*-
import os.path
auth_url = 'https://example.org/auth/v1.0'
username = 'username'
password = 'password'
token = 'hoge'
s_url = 'fuga'
container_name = 'moge'
objects = [{u'bytes': 5246,
            u'content_type': u'text/plain',
            u'hash': u'd226a3f396cbe3c187f2b7b78030eebb',
            u'last_modified': u'2013-05-01T08:40:00.459930',
            u'name': u'sample.txt'},
           {u'bytes': 5246,
            u'content_type': u'text/plain',
            u'hash': u'd226a3f396cbe3c187f2b7b78030eebb',
            u'last_modified': u'2013-05-01T09:23:40.060560',
            u'name': u'sample_2.txt'},
           {u'bytes': 5246,
            u'content_type': u'text/plain',
            u'hash': u'd226a3f396cbe3c187f2b7b78030eebb',
            u'last_modified': u'2013-05-01T09:23:26.092580',
            u'name': u'sample_3.txt'}]
objects_name_l = [i.get('name') for i in objects]
test_file = os.path.abspath('src/backup2swift_tests/sample.txt')
object_name = 'sample.txt'



# -*- coding: utf-8 -*-
"""
    Copyright (C) 2013 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import sys

from setuptools import setup, find_packages

sys.path.insert(0, 'src')
import backup2swift

classifiers = [
    "Development Status :: 3 - Alpha",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
    "Programming Language :: Python",
    "Topic :: Internet",
    "Topic :: Internet :: WWW/HTTP",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Environment :: OpenStack",
]

long_description = \
        open(os.path.join("docs","README.rst")).read() + \
        open(os.path.join("docs","TODO.rst")).read() + \
        open(os.path.join("docs","HISTORY.rst")).read()

if sys.version_info > (2, 6) and sys.version_info < (2, 7):
    requires = ['setuptools', 'swiftsc', 'argparse']
elif sys.version_info > (2, 7):
    requires = ['setuptools', 'swiftsc']

setup(name='backup2swift',
      version=backup2swift.__version__,
      description='Backup data to OpenStack Swift',
      long_description=long_description,
      author='Kouhei Maeda',
      author_email='mkouhei@palmtb.net',
      url='https://github.com/mkouhei/backup2swift',
      license=' GNU General Public License version 3',
      classifiers=classifiers,
      packages=find_packages('src'),
      package_dir={'': 'src'},
      data_files=[('share/backup2swift/examples', ['examples/bu2sw.conf', 'examples/bu2sw_ignore_verify.conf'])],
      install_requires=requires,
      extras_require=dict(
        test=[
            'pytest',
            'pep8',
            'mock',
            ],
        ),
      test_suite='tests',
      tests_require=['pytest','pep8','mock'],
      entry_points={
        "console_scripts": [
            "bu2sw = backup2swift.command:main",
            ]
        },
)
